from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="Phoneme")


@_attrs_define
class Phoneme:
    """
    Attributes:
        value (str): The actual phonetic transcription in the NT-SAMPA format.
        language (str | Unset): The [BCP 47](https://en.wikipedia.org/wiki/IETF_language_tag) language code.
        preferred (bool | Unset): Whether or not it is the preferred phoneme.
    """

    value: str
    language: str | Unset = UNSET
    preferred: bool | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        value = self.value

        language = self.language

        preferred = self.preferred

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "value": value,
        })
        if language is not UNSET:
            field_dict["language"] = language
        if preferred is not UNSET:
            field_dict["preferred"] = preferred

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        value = d.pop("value")

        language = d.pop("language", UNSET)

        preferred = d.pop("preferred", UNSET)

        phoneme = cls(
            value=value,
            language=language,
            preferred=preferred,
        )

        phoneme.additional_properties = d
        return phoneme

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
