from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="MaxSpeedOnSegmentPostInner")


@_attrs_define
class MaxSpeedOnSegmentPostInner:
    """
    Attributes:
        segment (str | Unset): Identifier of the segment with restrictions on maximum `baseSpeed`.

            Each entry has the following structure:
            `{segmentId}(#{direction})?`
        speed (float | Unset): Maximum `baseSpeed` on segment in m/s.
    """

    segment: str | Unset = UNSET
    speed: float | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        segment = self.segment

        speed = self.speed

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if segment is not UNSET:
            field_dict["segment"] = segment
        if speed is not UNSET:
            field_dict["speed"] = speed

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        segment = d.pop("segment", UNSET)

        speed = d.pop("speed", UNSET)

        max_speed_on_segment_post_inner = cls(
            segment=segment,
            speed=speed,
        )

        max_speed_on_segment_post_inner.additional_properties = d
        return max_speed_on_segment_post_inner

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
